@@ -1,5 +1,5 @@
-;; sdom.scm: main module exports and implementations for SDOM
-;; Copyright (C) 2010 Julian Graham
+;; core.scm: main module exports and implementations for SDOM
+;; Copyright (C) 2011 Julian Graham
 
 ;; SDOM free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 
 #!r6rs
 
-(library (sdom core (0 5))
+(library (sdom core (0 5 1))
   (export sdom:sxml->sdom 
 	  sdom:xml->sdom 
 
@@ -146,6 +146,7 @@
 	  sdom:user-data-event-node-renamed 
 	  sdom:user-data-event-node-adopted 
 
+	  sdom:dispatch-event
 	  sdom:register-event-dispatcher!
 
 	  sdom:register-feature! 
@@ -290,6 +291,7 @@
 		       (update-text-content! target)
 		       (set! text-content-roots 
 			     (cons target text-content-roots))))))
+		     
 	      ((whole-text) 
 	       (update-whole-text! (dom-synchronization-target cl))))
 	    (process-synchronizations-inner (cdr lst)))))
@@ -519,24 +521,6 @@
 		    (not (equal? prefix "xmlns"))) #f)
 	      (else #t)))))
     
-  (define (sdom:set-prefix! node p)
-    (let* ((namespace-uri (sdom:namespace-uri node))
-	   (name (sdom:local-name node)))
-      (if (or (not namespace-uri)
-	      (and (equal? p "xml")
-		   (not (equal? namespace-uri xml-ns-uri)))
-	      (and (sdom:attr? node)
-		   (or (and (equal? p "xmlns")
-			    (not (equal? namespace-uri xmlns-ns-uri)))
-		       (equal? (sdom:qualified-name node) "xmlns"))))
-	  (raise (sdom:make-exception sdom:exception-code-namespace-err #f))
-	  (cond ((sdom:attr? node)
-		 (set-car! (cdr node)
-			   (string->symbol (string-append (p ":" name)))))
-		((sdom:element? node)
-		 (set-car! (cdr node)
-			   (string->symbol (string-append (p ":" name)))))))))
-
   (define-condition-type &sdom:exception &condition 
     sdom:make-exception sdom:exception?
     (exception-code sdom:exception-code)
@@ -1090,7 +1074,7 @@
 		     ((eq? sym 'ns)
 		      (if (and ns (equal? str prefix)) 
 			  ns
-			  (let ((decl (lookup-namespace-at-node node str)))
+			  (let ((decl (sdom:lookup-namespace-uri node str)))
 			    (if decl
 				(cadr decl)
 				(let ((ae (aef node)))
@@ -1114,7 +1098,7 @@
 		 (eqv? type sdom:node-type-document-type)
 		 (eqv? type sdom:node-type-document-fragment)) #f)
 	    ((eqv? type sdom:node-type-attr)
-	     (let ((p (hashtable-ref parent-node-hash (cdr node) #f)))
+	     (let ((p (sdom:parent-node node)))
 	       (if p (internal-ns-lookup p str sym) #f)))
 	    (else (let ((ae (aef node)))
 		    (if ae (internal-ns-lookup ae str sym) #f))))))
@@ -1142,13 +1126,14 @@
 		   (if (null? as) 
 		       (lookup-namespace-uri 
 			(match-ancestor sdom:element? node))
-		       (cond ((and (equal? (sdom:prefix a) "xmlns")
-				   (equal? (sdom:local-name a) prefix))
-			      (sdom:value a))
-			     ((and (equal? (sdom:local-name a) "xmlns")
-				   (not (sdom:prefix a)))
-			      (sdom:value a))
-			     (else (loop (cdr as))))))))
+		       (let ((a (car as)))
+			 (cond ((and (equal? (sdom:prefix a) "xmlns")
+				     (equal? (sdom:local-name a) prefix))
+				(sdom:value a))
+			       ((and (equal? (sdom:local-name a) "xmlns")
+				     (not (sdom:prefix a)))
+				(sdom:value a))
+			       (else (loop (cdr as)))))))))
 	    ((sdom:document? node)
 	     (lookup-namespace-uri (sdom:document-element node)))
 	    ((or (sdom:entity? node)
@@ -1288,16 +1273,6 @@
 	    (sdom:processing-instruction? node)))
       (if (null? queue) (begin (merge-text!) lst)
 	  (let ((q (car queue)))
-#|
-	    (display "normalization; head is ")
-	    (display (car q))
-	    (display " ")
-	    (display (sdom:node-name (cadr q)))
-	    (display " => ")
-	    (display (sdom:node-value (cadr q)))
-	    (newline)
-	    (newline)
-|#	    
 	    (case (car q)
 	      ((enter)
 	       (let ((node (cadr q)))
@@ -1390,13 +1365,6 @@
 		((enter)
 		 (let* ((node (cadr q))
 			(parent (sdom:parent-node node)))
-		   #|	      (display "normalization; head is ")
-		   (display (sdom:node-name node))
-		   (display " => ")
-		   (display (sdom:node-value node))
-		   (newline)
-		   (newline)
-		   |#
 
 		   (cond ((sdom:cdata-section? node)
 			  (cond ((and split-cdata-sections?
@@ -1623,15 +1591,6 @@
 
       (if (null? queue) lst
 	  (let ((q (car queue)))
-#|	    (display "head is ")
-	    (display (car q))
-	    (display " ")
-	    (display (sdom:node-name (cadr q)))
-	    (display " => ")
-	    (display (sdom:node-value (cadr q)))
-	    (newline)
-	    (newline)
-|#
 	    (case (car q)
 	      ((enter)
 	       (let ((node (cadr q)))
@@ -1675,7 +1634,7 @@
 		     (if (string? ca) 
 			 ca 
 			 (raise (sdom:make-exception 
-				 exception-code-type-mismatch-err #f)))))))
+				 sdom:exception-code-type-mismatch-err #f)))))))
       (if (not (sdom:document? doc))
 	  (raise (sdom:make-exception sdom:exception-code-type-mismatch-err 
 				      #f)))
@@ -1722,8 +1681,6 @@
 	    (sdom:set-specified! new-node #t)
 	    (sdom:set-is-id! new-node #f)))
       (sdom:set-owner-document! new-node doc)
-      (handle-user-data-event 
-       node sdom:user-data-event-node-imported node new-node)
 
       (if (sdom:element? node)
 	  (let ((child-attrs (map (lambda (x) (import-node doc x #t))
@@ -1743,7 +1700,11 @@
   (define (sdom:import-node doc node deep)
     (if (or (sdom:document? node) (sdom:document-type? node))
 	(raise (sdom:make-exception sdom:exception-code-not-supported-err #f))
-	(with-synchronizations (import-node doc node deep))))
+	(with-synchronizations 
+	 (let ((new-node (import-node doc node deep)))
+	   (handle-user-data-event 
+	    node sdom:user-data-event-node-imported node new-node)
+	   new-node))))
 
   ;; INCOMPLETE -- NEED TO ADD NEW NAMESPACE DECL IF NECESSARY!
 
@@ -1759,7 +1720,6 @@
     (let ((parent (sdom:parent-node node)))
       (if (and parent orphan?) (sdom:remove-child! parent node)))
     (sdom:set-owner-document! node doc)
-    (handle-user-data-event node sdom:user-data-event-node-adopted node '())
     node)
 
   (define (sdom:adopt-node! doc node)
@@ -1773,7 +1733,9 @@
     (if (sdom:read-only node)
 	(raise (sdom:make-exception 
 		sdom:exception-code-no-modification-allowed-err #f)))
-    (adopt-node! doc node #t))
+    (adopt-node! doc node #t)
+    (handle-user-data-event node sdom:user-data-event-node-adopted node '())
+    node)      
 
   (define (sdom:rename-node! node qname ns)
     (let ((old-name (string-copy (sdom:node-name node)))
@@ -1854,24 +1816,22 @@
        (sdom:set-attribute-node-internal! elt node old-node))
       (sdom:dispatch-event elt "DOMAttrModified" node #f new-value name 2)))
 
-  (define (remove-attribute! elt pos)
+  (define (remove-attribute! elt node)
     (let* ((attributes (sdom:attributes elt))
-	   (attribute (list-ref attributes pos))
-	   (attributes (append (list-head attributes pos)
-			       (list-tail attributes (+ pos 1)))))
+	   (attributes (remq node attributes)))
 
-      (sdom:set-attributes! elt attributes)
-      (orphan-node! attribute)
+      (sdom:set-attributes! elt node)
+      (orphan-node! node)
       
-      attribute))
+      node))
 
   (define (sdom:remove-attribute-node! elt node) 
     (if (not (and (sdom:element? elt) (sdom:attr? node)))
 	(raise (sdom:make-exception sdom:exception-code-type-mismatch-err #f)))
     (if (not (sdom:same-node? elt (sdom:owner-element node)))
 	(raise (sdom:make-exception sdom:exception-code-not-found-err #f)))
-    
-    (delq! (cdr node) (single-at-finder elt)))
+
+    (remove-attribute! elt node))
 
   (define (sdom:get-attribute elt name . namespace-uri)
     (let ((node (if (not (null? namespace-uri))
@@ -1881,7 +1841,7 @@
 
   (define (sdom:set-attribute! elt name value . namespace-uri)
     (if (not (sdom:element? elt))
-	(raise (sdom:make-exceptiom sdom:exception-code-type-mismatch-err #f)))
+	(raise (sdom:make-exception sdom:exception-code-type-mismatch-err #f)))
     (let ((ns (if (not (null? namespace-uri)) (car namespace-uri) #f))
 	  (doc (sdom:owner-document elt)))
       (if (not (valid-namespace-combo? doc name ns))
@@ -1903,15 +1863,13 @@
   (define (update-text-content! root)
     (define (update-text-content-visitor continuation queue lst)
       (define (update-node! node)
-	(sdom:set-text-content-internal!
+	(sdom:set-text-content-internal! 
 	 node (apply string-append
 		     (map (lambda (n) (or (sdom:text-content n) "")) 
 			  (filter (lambda (n)
 				    (not (or (sdom:comment? n)
 					     (sdom:processing-instruction? n))))
 				  (sdom:child-nodes node))))))
-
-
       (if (null? queue) lst
 	  (let ((q (car queue)))
 	    (if (eq? (car q) 'exit)
@@ -1930,12 +1888,6 @@
 			  node (sdom:node-value node))))))
 	    (continuation queue lst))))
 
-;      (display "updating text content on ")
-;      (display (sdom:node-name root))
-;      (display " => ")
-;      (display (sdom:node-value root))
-;      (newline)
-
     (document-order root update-text-content-visitor))
 
   (define (set-value! attr value)
@@ -2100,7 +2052,7 @@
 		  (proc (make-prepender entity)))
 	     (apply values (append `(,entity)
 				   `((process ,(caddr node) ,proc))))))
-	  ((*ENTITY-REF*) (values (sdom:make-entity-ref)))
+	  ((*ENTITY-REF*) (values (sdom:make-entity-reference)))
 	  ((*FRAGMENT*) (values (sdom:make-document-fragment)))
 	  ((*DOCTYPE*) 
 	   (let ((dt (sdom:make-document-type 
@@ -2108,13 +2060,13 @@
 		      (list-ref node 4)
 		      (list-ref node 5)
 		      (list-ref node 6))))
-	   (apply values
-		  (append (list dt)
-			  (let ((ents (list-ref node 2))
-				(proc (make-document-type-prepender dt)))
-			    (if (null? ents)
-				'() (map (lambda (e) (list 'process e proc))
-					 ents)))))))
+	     (apply values
+		    (append (list dt)
+			    (let ((ents (list-ref node 2))
+				  (proc (make-document-type-prepender dt)))
+			      (if ents
+				  (map (lambda (e) (list 'process e proc)) ents)
+				  '()))))))
 
 	  ((*PI*) (values (sdom:make-processing-instruction 
 			   (symbol->string (cadr node)) (caddr node))))
@@ -2155,8 +2107,12 @@
 	  ((postprocess) (let-values (((proc node) (apply values (cdr op))))
 			   (proc node)
 			   (process-tree (cdr operations))))
-	  (else (error)))))
+	  (else (raise (condition 
+			(make-assertion-violation)
+			(make-message-condition 
+			 "Unknown parsing operation.")))))))
 
-    (let-values (((doc . child-ops) (process-node sxml-tree)))
-      (process-tree (append child-ops `((done ,doc))))))
+    (with-synchronizations
+     (let-values (((doc . child-ops) (process-node sxml-tree)))
+       (process-tree (append child-ops `((done ,doc)))))))
 )
